import { classes } from '@automapper/classes';
import { AutomapperModule } from '@automapper/nestjs';
import { Test, TestingModule } from '@nestjs/testing';
import { ICollection } from 'src/modules/database/src/interfaces/collection.interface';
import { ConfigurationMessager } from '../../adapters/secondaries/configuration.messager';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { LoggingMessager } from '../../adapters/secondaries/logging.messager';
import { DeleteConfigurationCommand } from '../../commands/delete-configuration.command';
import { Domain } from '../../domain/dtos/domain.enum';
import { Configuration } from '../../domain/entities/configuration';
import { DeleteConfigurationUseCase } from '../../domain/usecases/delete-configuration.usecase';
import { ConfigurationProfile } from '../../mappers/configuration.profile';

const mockConfigurations: ICollection<Configuration> = {
  data: [
    {
      uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
      domain: Domain.USER,
      key: 'key1',
      value: 'value1',
    },
    {
      uuid: 'bb281075-1b98-4456-89d6-c643d3044a92',
      domain: Domain.USER,
      key: 'key2',
      value: 'value2',
    },
    {
      uuid: 'bb281075-1b98-4456-89d6-c643d3044a93',
      domain: Domain.USER,
      key: 'key3',
      value: 'value3',
    },
  ],
  total: 3,
};

const mockConfigurationRepository = {
  delete: jest
    .fn()
    .mockImplementationOnce((uuid: string) => {
      let savedConfiguration = {};
      mockConfigurations.data.forEach((configuration, index) => {
        if (configuration.uuid === uuid) {
          savedConfiguration = { ...configuration };
          mockConfigurations.data.splice(index, 1);
        }
      });
      return savedConfiguration;
    })
    .mockImplementation(() => {
      throw new Error('Error');
    }),
};

const mockConfigurationMessager = {
  publish: jest.fn().mockImplementation(),
};

const mockLoggingMessager = {
  publish: jest.fn().mockImplementation(),
};
describe('DeleteConfigurationUseCase', () => {
  let deleteConfigurationUseCase: DeleteConfigurationUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [AutomapperModule.forRoot({ strategyInitializer: classes() })],
      providers: [
        {
          provide: ConfigurationRepository,
          useValue: mockConfigurationRepository,
        },
        DeleteConfigurationUseCase,
        ConfigurationProfile,
        {
          provide: ConfigurationMessager,
          useValue: mockConfigurationMessager,
        },
        {
          provide: LoggingMessager,
          useValue: mockLoggingMessager,
        },
      ],
    }).compile();

    deleteConfigurationUseCase = module.get<DeleteConfigurationUseCase>(
      DeleteConfigurationUseCase,
    );
  });

  it('should be defined', () => {
    expect(deleteConfigurationUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should delete a configuration', async () => {
      jest.spyOn(mockConfigurationMessager, 'publish');
      const savedUuid = mockConfigurations.data[0].uuid;
      const deleteConfigurationCommand = new DeleteConfigurationCommand(
        savedUuid,
      );
      await deleteConfigurationUseCase.execute(deleteConfigurationCommand);

      const deletedConfiguration = mockConfigurations.data.find(
        (configuration) => configuration.uuid === savedUuid,
      );
      expect(deletedConfiguration).toBeUndefined();
      expect(mockConfigurationMessager.publish).toHaveBeenCalledTimes(1);
    });
    it('should throw an error if configuration does not exist', async () => {
      await expect(
        deleteConfigurationUseCase.execute(
          new DeleteConfigurationCommand('wrong uuid'),
        ),
      ).rejects.toBeInstanceOf(Error);
    });
  });
});
