import { classes } from '@automapper/classes';
import { AutomapperModule } from '@automapper/nestjs';
import { Test, TestingModule } from '@nestjs/testing';
import { ConfigurationMessager } from '../../adapters/secondaries/configuration.messager';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { LoggingMessager } from '../../adapters/secondaries/logging.messager';
import { UpdateConfigurationCommand } from '../../commands/update-configuration.command';
import { Domain } from '../../domain/dtos/domain.enum';
import { UpdateConfigurationRequest } from '../../domain/dtos/update-configuration.request';
import { Configuration } from '../../domain/entities/configuration';
import { UpdateConfigurationUseCase } from '../../domain/usecases/update-configuration.usecase';
import { ConfigurationProfile } from '../../mappers/configuration.profile';

const originalConfiguration: Configuration = new Configuration();
originalConfiguration.uuid = 'bb281075-1b98-4456-89d6-c643d3044a91';
originalConfiguration.domain = Domain.USER;
originalConfiguration.key = 'key1';
originalConfiguration.value = 'value1';

const updateConfigurationRequest: UpdateConfigurationRequest = {
  uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
  value: 'value2',
};

const updateConfigurationCommand: UpdateConfigurationCommand =
  new UpdateConfigurationCommand(updateConfigurationRequest);

const mockConfigurationRepository = {
  update: jest
    .fn()
    .mockImplementationOnce((uuid: string, params: any) => {
      originalConfiguration.value = params.value;

      return Promise.resolve(originalConfiguration);
    })
    .mockImplementation(() => {
      throw new Error('Error');
    }),
};

const mockConfigurationMessager = {
  publish: jest.fn().mockImplementation(),
};

const mockLoggingMessager = {
  publish: jest.fn().mockImplementation(),
};

describe('UpdateConfigurationUseCase', () => {
  let updateConfigurationUseCase: UpdateConfigurationUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [AutomapperModule.forRoot({ strategyInitializer: classes() })],
      providers: [
        {
          provide: ConfigurationRepository,
          useValue: mockConfigurationRepository,
        },
        UpdateConfigurationUseCase,
        ConfigurationProfile,
        {
          provide: ConfigurationMessager,
          useValue: mockConfigurationMessager,
        },
        {
          provide: LoggingMessager,
          useValue: mockLoggingMessager,
        },
      ],
    }).compile();

    updateConfigurationUseCase = module.get<UpdateConfigurationUseCase>(
      UpdateConfigurationUseCase,
    );
  });

  it('should be defined', () => {
    expect(updateConfigurationUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should update a configuration value', async () => {
      jest.spyOn(mockConfigurationMessager, 'publish');
      const updatedConfiguration: Configuration =
        await updateConfigurationUseCase.execute(updateConfigurationCommand);

      expect(updatedConfiguration.value).toBe(updateConfigurationRequest.value);
      expect(mockConfigurationMessager.publish).toHaveBeenCalledTimes(1);
    });
    it('should throw an error if configuration does not exist', async () => {
      await expect(
        updateConfigurationUseCase.execute(updateConfigurationCommand),
      ).rejects.toBeInstanceOf(Error);
    });
  });
});
